//---------------------------------------------------------------------------
#ifndef LoggerH
#define LoggerH
//---------------------------------------------------------------------------
/*
XPERT LOG

Version 1.0, Jonathan D. Weisberg     (C)1999 Sutron Corp

Implements a wrap-around time stamped data store called LOG, and CURSOR class for navigating the log.

FEATURES:
o All data is compressed using LZO compression.
o Stores binary or ascii data of variable length up to (MaxDataSize)
o Access data by line number, or search by time stamp.
o Space is pre-allocated and never exceeded.
o Stores all data in 1 or more sub-blocks (SubBlockSize) to make it easy and quick to find data.
o Reads the size of data blocks and sub blocks from the LOG file itself to support future tweaks to this sizes.
o Access to the log thru the class is thread-safe.
o Compressed data is validated with a CRC16
o Error recovery is possible since one bad block does not effect the ability to read other blocks.

09/27/1999 JDW Fixed a problem with Moving a cursor without setting it to the top or bottom first.
Also added some new versions of AppendSensor and ReadSensor which support
ASCII storage of the data values. The binary version still exists too,
but only one type should be stored in a log.
*/


#ifdef __BORLANDC__
#define LOGDECL
#else
#ifdef LOGGER_DLL
#define LOGDECL _declspec(dllexport)
#else
#define LOGDECL _declspec(dllimport)
#endif
#endif

#ifdef __BORLANDC__
typedef char TCHAR;
#else
#include <windows.h>
#endif

typedef short int INT16;
typedef unsigned short int UINT16;
typedef __int64 INT64;
typedef unsigned __int64 UINT64;
typedef unsigned char UINT8;
typedef int INT32;
typedef unsigned int UINT32;

#include "LogFlags.h"

extern "C" LPCTSTR LOGDECL GetLoggerVersion();
extern "C" LPCTSTR LOGDECL GetLoggerPath();

/*
Routines to Open/Create and Append data to an XPERT LOG File.
*/


class LOGDECL LOG
{
public:
   LOG();
   ~LOG();

   bool Create(LPCTSTR Name, LPCTSTR Memo, int Size,    // Creates a log file of the given name with the given size in bytes,
      UINT32 LogFlags);                                 // Various log options 

   bool Create(LPCTSTR Name, LPCTSTR Memo, int Size,    // Creates a log file but with extra control over the block sizes that control how data is
      int SubBlockSize,                                 // stored. SubBlockSize is 128 by default. Decreasing this will decrease overhead, but will
      int DesiredDataSize,                              // increase processing. DesiredDataSize is 1910 by default, if appending new data to a block
      int MaxDataSize,                                  // exceeds this limit, the block is written out. MaxDataSize is 2048 by default and determines
                                                        // the largest line or combinination of lines of data that can be read or written to the log.
      UINT32 LogFlags);                                 // Various log options.  

   bool Resize(LPCTSTR Name, LPCTSTR Memo, int Size, UINT32 Flags);
                                                        // Resizes the log, growing or shrinking as needed.

   void Flush(bool Force=true);                         // Flushes unwritten data out to disk if force is set to true, otherwise will only
                                                        // write out data if a timer is exceeded.

   void SaveState();                                    // Flushes data and saves the log's state information required to quickly open the file

   bool Open(LPCTSTR Name);                             // Opens an existing log file.

   void Close();                                        // Closes a log file.

   bool Append(INT64 Time, void* Data, int Len);        // Appends aribtrary binary data to a log file and time stamps it with the given Time.

   bool AppendNote(INT64 Time, char* Note);            // Appends a note to the log

   bool AppendNote(INT64 Time, LPCTSTR Note);          // Appends a note to the log

   bool AppendRecord(INT64 Time, LPCTSTR Id, LPCTSTR Record);
                                                       // Appends a record to the log

   bool AppendSensor(const INT64 Time, LPCTSTR Name,   // Appends sensor data to the log
      LPCTSTR Units, UINT8 Quality,                    // This version double ASCII data
      const double Num);                               // Only use one version of append sensor in a log!

   bool AppendSensor(const INT64 Time, LPCTSTR Name,   // Appends sensor data to the log
      LPCTSTR Units, UINT8 Quality,                    // This version uses ASCII data
      LPCTSTR Num);                                    // Only use one version of append sensor in a log!

   bool Changed(UINT32& Counter);                      // Returns true if the log has changed since the last time changed has been called.
                                                       // Counter is a variable passed in which is used to compare to the logs internal change
                                                       // counter. If the two counters differ then a change is detected, and the new value is
                                                       // returned in Counter.
 
   bool ReadExtra(PVOID Extra, int Size);             // Reads application specific data stored at the end of the log.

   bool WriteExtra(PVOID Extra, int Size);            // Writes application specific data stored at the end of the log.

   UINT32 GetFlags();                                 // Returns the LogFlags see "LogFlag.h"

   UINT32 GetExtraSize();                             // Returns the number of bytes of extra data which are stored at the end of the log.

   UINT32 GetSize();                                  // Returns the total number of bytes stored in the log file

   UINT32 GetLineCount();                             // Returns the number of lines in the log.

   void GetMemo(TCHAR* Memo);                         // Stores the log's memo field in pre-allocted wide char string
   // of at least 64 bytes.

   void SetMemo(LPCTSTR Memo);                        // Set's the log's memo field to the passed-in string.

   void EraseData();                                  // Erases all log data, log must already be open.

   static bool IsValidLogFile(LPCTSTR Name);          // Returns true if name is a valid .log file

   bool BeingAccessed();                              // True if the log is being accessed and should not be deleted

   bool BeingClosed();                                // True if the log is in the process of being closed

   void Closing();                                    // Indicate that the log will be closed as soon as it's no longer being accessed
private:
   friend class LOGCURSOR;
   PVOID Log;
};

/*
   Routines to Read and Navigate an XPERT LOG File.
   Once a position in the log file is established, it is maintained even after
   lines have been added or deleted from the file. If the record the cursor points
   to is delete then the cursor is set to the top.
   The top refers to oldest data, and bottom to newest.
*/
class LOGDECL LOGCURSOR
{
public:
   LOGCURSOR(LOG& log);
   ~LOGCURSOR();

   bool GotoTop();                                 // Move to the oldest data in the log

   bool GotoBottom();                              // Move to the newest data in the log

   bool MoveTo(int AbsLine);                       // Go to a line in the log,
                                                   // returns false if a move before the beginning of the log
                                                   // or after the end of the log occurs.

   bool MoveBy(int RelLine);                       // Move a relative number of lines in the log,
                                                   // returns false if a move before the beginning of the log
                                                   // or after the end of the log occurs.

   bool MoveNext();                                // Moves to the next (newer) line in the log, skipping any bad blocks

   bool MovePrev();                                // Moves to the previous (older) line in the log, skipping any bad blocks

   bool Search(INT64 Time);                        // Search backwards from the end of the log for the first data
                                                   // which is at or before the passed Time. If data is found having 
                                                   // equal time, the cursor is set to point to it. Otherwise, if 
                                                   // data is found with a prior time, the cursor is set to point to 
                                                   // the data just after it. Otherwise, false is returned and the 
                                                   // cursor position is set to the top.

   bool Read(INT64& Time, PCHAR& Data, int& Len);  // Reads log data at the current position. The data pointer returned
                                                   // is valid as long as the cursor is not moved, and points in to data
                                                   // pre-allocated by the cursor class.

   bool IsNote();                                  // Returns true if the current record is a note, false if it contains
                                                   // Data. Always call this first in order to determine whether to call
                                                   // ReadNote() or ReadSensor() next (also check IsRecord())

   bool IsRecord();                                // Returns true if the current record is a an entire string of data
                                                   // Always call this and IsNote() to tell how to process the data


   bool ReadNote(INT64& Time, TCHAR* Note);        // Read a log note from the current record.
                                                   // Always call IsNote() before calling this.
                                                   // The caller must allocate space for the note, the size depends
                                                   // on the longest note Appended.

   bool ReadRecord(INT64& Time, TCHAR* Id, TCHAR* Record);
                                                   // Reads a log record from the current position
                                                   // Always call IsRecord() before calling this.
                                                   // The caller must allocate space, max of Id[33], and Record[1001]

   INT64 ReadTime();                               // Reads just the current time of the current line, if the current
                                                   // line is invalid, then it returns the base time of the bad block
                                                   // which contains the line.

   bool ReadSensor(INT64& Time, TCHAR* Name,       // Reads sensor data from the current record
      TCHAR* Units, UINT8& Quality,                // Always call IsNote() before calling this.
      double& Num);                                // The caller must allocate space for the Name and Units, the size
                                                   // depends on the longest Names & Units stored in the log.

   bool ReadSensor(INT64& Time, TCHAR* Name,       // Reads sensor data from the current record
      TCHAR* Units, UINT8& Quality,                // Always call IsNote() before calling this.
      TCHAR* Num);                                 // The caller must allocate space for the Name, Units and Num, the size
                                                   // depends on the longest Names & Units stored in the log.

   bool AtTop();                                   // Detects whether the current line is the first line in the log.

   bool AtBottom();                                // Detects whether the current line is the last line in the log.

   int GetCurrentLine();                           // Returns the line number the cursor is position at, 0..LineCount-1

   void Sync();                                    // Synchronizes the Cursor with the Log, the Move commands do this
                                                   // Automatically but other commands do not.

// Versions of the standard functions that return time truncated to the second as opposed to milli-second resolution

   bool ReadSec(INT64& Time, PCHAR& Data, int& Len); // Reads log data at the current position. The data pointer returned
                                                     // is valid as long as the cursor is not moved, and points in to data
                                                     // pre-allocated by the cursor class.

   bool ReadNoteSec(INT64& Time, TCHAR* Note);      // Read a log note from the current record.
                                                   // Always call IsNote() before calling this.
                                                   // The caller must allocate space for the note, the size depends
                                                   // on the longest note Appended.

   bool ReadRecordSec(INT64& Time, TCHAR* Id, TCHAR* Record);
                                                   // Reads a log record from the current position
                                                   // Always call IsRecord() before calling this.
                                                   // The caller must allocate space, max of Id[33], and Record[1001]

   INT64 ReadTimeSec();                               // Reads just the current time of the current line, if the current
                                                   // line is invalid, then it returns the base time of the bad block
                                                   // which contains the line.

   bool ReadSensorSec(INT64& Time, TCHAR* Name,    // Reads sensor data from the current record
      TCHAR* Units, UINT8& Quality,                // Always call IsNote() before calling this.
      double& Num);                                // The caller must allocate space for the Name and Units, the size
                                                   // depends on the longest Names & Units stored in the log.

   bool ReadSensorSec(INT64& Time, TCHAR* Name,    // Reads sensor data from the current record
      TCHAR* Units, UINT8& Quality,                // Always call IsNote() before calling this.
      TCHAR* Num);                                 // The caller must allocate space for the Name, Units and Num, the size

   bool BeingClosed();                                // True if the log is in the process of being closed

private:
   PVOID Cursor;
};


#endif
