// Semaphore.h: interface for the TSemaphore class.
//

#if !defined(_TSEMAPHORE_H_)
#define _TSEMAPHORE_H_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#undef DECL
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif

// TCriticalSection is a wrapper for CRITICAL_SECTION
class DECL TCriticalSection
{
public:
   TCriticalSection()  { InitializeCriticalSection(&m_csLock); };
   ~TCriticalSection() { DeleteCriticalSection(&m_csLock); };
   void Lock()         { EnterCriticalSection(&m_csLock); };
   bool LockTry()      { return TryEnterCriticalSection(&m_csLock) == TRUE; };
   void Unlock()       { LeaveCriticalSection(&m_csLock); };
private:
   CRITICAL_SECTION m_csLock;
};

// Locks a TCriticalSection on instantiation, and automatically Unlocks it upon destruction
class DECL TCriticalLock
{
public:
   TCriticalLock(TCriticalSection& section) : m_pSection(&section) { section.Lock();                                          };
   void Unlock()                                                   { if (m_pSection) m_pSection->Unlock(); m_pSection = NULL; };
   ~TCriticalLock()                                                { Unlock();                                                };
private:
   TCriticalSection* m_pSection;
};


// TSemaphore and TSingleLock are used to implement timed waits 
// (functionality that is normally provided by CSemaphore or CMutex 
// and CSingleLock, but which don't exist in our WCE build).

// Unlike event handles, which release all threads when a manual-reset 
// event is signaled, when a mutex is signaled, only one thread is 
// released. 


//////////////////////////////////////////////////////////////////////
// TSemaphore Class
//
class DECL TSemaphore
{
   // TSingleLock is typically used to lock and unlock on the stack.
   friend class TSingleLock;
   
   // Constructors and destructor.
   public:
   TSemaphore();
   TSemaphore(const TCHAR* szName);
   virtual ~TSemaphore();

   // Methods
   public:
   bool Lock(DWORD dwWait=INFINITE) 
         {return (WAIT_OBJECT_0 == WaitForSingleObject(m_hSem, dwWait));}
   bool Unlock() {return (FALSE != ReleaseMutex(m_hSem));}

   // Attributes
   protected:
   HANDLE m_hSem;
};


//////////////////////////////////////////////////////////////////////
// TSingleLock Class
//
class DECL TSingleLock
{
   // Constructors and destructor.
   public:
   TSingleLock(TSemaphore& sem, bool bUnlockOnDestroy = true) : m_sem(sem) 
   {
      m_bAutoUnlock = bUnlockOnDestroy;
      m_bOwn = false;
   };
   virtual ~TSingleLock() {if (m_bAutoUnlock && m_bOwn) m_sem.Unlock();};

   // Methods
   public:
   bool Lock(DWORD dwWait=INFINITE) {return (m_sem.Lock(dwWait) ? m_bOwn = true : false);};
   bool Unlock() {return m_sem.Unlock();};

   // Attributes
   protected:
   TSemaphore& m_sem;
   bool m_bAutoUnlock;
   bool m_bOwn;
};


//////////////////////////////////////////////////////////////////////
// TRWLock Class
//
class DECL TRWLock
{
   // This class is used to control read and write access to a resource
   // and has fairly high overhead in order to achieve flexibility and
   // avoid common deadlock scenarios. 
   //
   // When a write is active, only the same thead can obtain a read or
   // write access.
   //
   // When a read is active, other theads can obtain read access as well,
   // but only one thread can also obtain write access.
   //
   // What this permits is for a thread to obtain read access to a resource
   // and then later obtain write access, assuming no other threads have
   // requested read access. In addition a thread can obtain write access
   // to a resouce and then later obtain read access. No other thread can
   // access the list in that scenario.
   //
   // Recursion is supported. A thread can obtain the same lock multiple
   // times. 

   // Constructors and destructor.
   public:
   TRWLock();
   virtual ~TRWLock();

   // Methods
   public:
   bool LockRead(DWORD dwWait = INFINITE);
   void UnlockRead();
   bool LockWrite(DWORD dwWait = INFINITE);
   void UnlockWrite();

   // Attributes
   protected:
 	typedef CMap<DWORD, DWORD, DWORD, DWORD> CMapDWordToDWord;

   // Reader event, signaled when a reader has finished and a write may be allowed
   HANDLE m_hCheckWrite;

   // ThreadId of a thread that currently has sole read access
   DWORD m_dwReadyThread;
   
   // Write lock.
   TSemaphore m_semWrite;

   // Synchronizes access to read counter.
   CRITICAL_SECTION m_cs;

   // Tracks the IDs of threads with read access to the list
   CMapDWordToDWord m_ReaderList;
};


//////////////////////////////////////////////////////////////////////
// TLock Class
//
class DECL TLock
{
   // This is a simple read/write lock class. It blocks all other access when the write
   // lock is asserted, and won't grant write access until all read locks are clear.
   // Allows n-readers and 1 writer. Nested access is allowed, which means
   // a thread which calls a function that does a LockWrite() can call a function which
   // does a LockRead() and the thread won't block. Nested LockWrite's and nested LockRead's
   // are also permitted.

   // Constructors and destructor.
public:
   TLock();
   ~TLock();

   // Methods
public:
   void LockRead();
   void UnLockRead();
   bool LockWrite(DWORD Timeout=INFINITE);
   void UnLockWrite();
   HANDLE NoReaders; // Is true when there are no readers and we can potentially write

protected:
   int Count;
   CRITICAL_SECTION Lock;
};


#endif // !defined(_TSEMAPHORE_H_)
