//
// Users.h
//

#ifndef _Users_H_
#define _Users_H_


#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
   #error include 'stdafx.h' before including this file for PCH
#endif

#include <afxtempl.h>         // For CTypedPtrArray

#include "Utils/Semaphore.h"
#include "Utils/Time.h"

#undef DECL
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif


/*///////////////////////////////////////////////////////////////////////////

1. An instance of the class CUsers is used to contain the set of users and 
   their associated login attributes. The data is loaded from the user data 
   file on construction. The data is written back to file on destruction, 
   or it can be written on request by calling Commit()
2. Instances of the class CUser are used by CUsers to contain single user 
   accounts. 
3. The user data file is a text file containing a single user account per 
   line. Each account has four fields: name, password, user-group, and timeout.
   The field delimiter is the character '|'. The character '#' is used to 
   indicate no password. Only characters present on the built-in keypad are 
   valid for use in names and passwords.
4. GetUser(CUser& u, int i) and GetUserCount() are subject to filtering based 
   on user group as set by SetFilter(). Note that the index supplied to 
   GetUser() is relative to filtered objects, e.g., GetUser(2) returns the 
   second user found subject to any applied filter. Note that the filter logic 
   is exclusive. For example, a filter of DATA_RETRIEVAL_MODE would filter out 
   all users except those with a user group of DATA_RETRIEVAL_MODE. 

///////////////////////////////////////////////////////////////////////////*/


///////////////////////////////////////////////////////////////////////////////
// TUserGroup - enum used to identify the current login mode CModPropPage
// accessibility. Note that CModPropPage accessibility is cumulative, such that
// higher mode values imply accessibility to modes of lower value.
//
// WARNING: This structure is used by Remote.cpp and XTermDlg.cpp to support 
//  custom groups and access level on login, so try not to do anything that would 
//  change the value of an existing group. If such a change is required be sure 
//  to update Remote.cpp as well and understand upwards compatibility will be 
//  sacrificed.
enum TUserGroup {DATA_RETRIEVAL_MODE, SETUP_MODE, 
      CUSTOM_GROUP1, CUSTOM_GROUP2, CUSTOM_GROUP3, CUSTOM_GROUP4, CUSTOM_GROUP5, 
      CUSTOM_GROUP6, CUSTOM_GROUP7, CUSTOM_GROUP8, CUSTOM_GROUP9, CUSTOM_GROUP10, 
      CUSTOM_SETUP_GROUP1, CUSTOM_SETUP_GROUP2, CUSTOM_SETUP_GROUP3, CUSTOM_SETUP_GROUP4, CUSTOM_SETUP_GROUP5, 
      CUSTOM_SETUP_GROUP6, CUSTOM_SETUP_GROUP7, CUSTOM_SETUP_GROUP8, CUSTOM_SETUP_GROUP9, CUSTOM_SETUP_GROUP10, 
      NUM_USER_GROUPS, NUM_CUSTOM_GROUPS=NUM_USER_GROUPS-CUSTOM_GROUP1};

enum TAccessRights {DENY_COMMANDLINE=1, DENY_SSP=2, DENY_TCPIP=4, DENY_GUI=8};

extern "C" DECL TCHAR* szUserGroups[NUM_USER_GROUPS];

typedef bool (* LPFNCUSTOMLOGIN)(TUserGroup Group, LPCTSTR ComPort, LPCTSTR UserName);
typedef bool (* LPFNCUSTOMCOMMANDPARSER)(TUserGroup Group, LPCTSTR ComPort, LPCTSTR UserName, LPCTSTR Command);
typedef bool (* LPFNCUSTOMCOMMANDPARSERV2)(TUserGroup Group, LPCTSTR ComPort, LPCTSTR UserName, LPCTSTR Command, CString& CommandToIssue);

bool DECL AddCustomGroup(TUserGroup Group, LPCTSTR Description, LPFNCUSTOMLOGIN LoginFunct);

bool DECL CallCustomGroup(LPCTSTR UserName, LPCTSTR Password, LPCTSTR ComPort);

bool DECL AddCustomCommandParser(TUserGroup Group, LPFNCUSTOMCOMMANDPARSER CommandParser);

bool DECL AddCustomCommandParserV2(TUserGroup Group, LPFNCUSTOMCOMMANDPARSERV2 CommandParserV2);

bool DECL CallCustomCommandParser(LPCTSTR UserName, LPCTSTR ComPort, LPCTSTR Command, CString& CommandToIssue);

extern "C" DECL CString RemoteUserName;
extern "C" DECL CString RemotePassword;
extern "C" DECL DWORD RemoteLoginTime;
extern "C" DECL TUserGroup CurrentGroup;


class DECL TLoginPort : public CObject
{
   public:
   TLoginPort(const CString& strUserName, const CString& strPort) 
      : m_UserName(strUserName), m_Port(strPort) { m_LoginTime = TTime::GetCurrentTime(); };
   ~TLoginPort() {};

   public:
   CString m_UserName;
   CString m_Port;
   TTime m_LoginTime;
};

/////////////////////////////////////////////////////////////////////////////
// CUser

class DECL CUser : public CObject
{
   friend class CUsers;
   CString m_strName;
   CString m_strPassword;
   TUserGroup m_ugGroup;
   unsigned int m_nTimeoutInterval;
   unsigned int m_nAccessRights;
   bool m_bLoggedIn;
   int m_nLoginCount;

   // Methods
   public:
   CUser();
   CUser& operator=(const CUser& u);
   CString GetName() { return m_strName; };
   CString GetPassword() { return m_strPassword; };
   TUserGroup GetEffectiveUserGroup();
   TUserGroup GetUserGroup() { return m_ugGroup; };
   unsigned int GetTimeoutInterval() { return m_nTimeoutInterval; };
   unsigned int GetAccessRights() { return m_nAccessRights; };
   // IsLoggedIn will return false as soon as any port logs outs with the given user. 
   // Use GetLoginCount() to see if multiple ports are logged in on the same user
   bool IsLoggedIn()    { return m_bLoggedIn; };  
   int GetLoginCount()  { return m_nLoginCount; };
};


/////////////////////////////////////////////////////////////////////////////
// TMapStringToRefUpperCase is a version of TMapStringToRef that uppercases all strings
template<class TYPE>
class TUtils_MapStringToRefUpperCase : public CMapStringToPtr
{
public:
   BOOL Lookup(CString K, TYPE*& V) const                      { K.MakeUpper(); return CMapStringToPtr::Lookup(K, (void *&) V);    };
   TYPE& operator[](CString K)                                 { K.MakeUpper(); return * (TYPE*&) CMapStringToPtr::operator[](K);  };
   void SetAt(CString K, TYPE& V)                              { K.MakeUpper(); CMapStringToPtr::SetAt(K, &V);                     };
   void GetNextAssoc(POSITION& P, CString& K, TYPE*& V) const  { CMapStringToPtr::GetNextAssoc(P, K, (void *&) V);                 };
   BOOL RemoveKey(CString K)                                   { K.MakeUpper(); return CMapStringToPtr::RemoveKey(K);              };
};

/////////////////////////////////////////////////////////////////////////////
// CUsers

class DECL CUsers
{
   // Constructors and destructor
   public:
   CUsers();
   ~CUsers();

   // Attributes
   private:
   CFile m_file;
   CTypedPtrArray<CObArray, CUser*> m_Users;
   bool m_bFilter;
   TUserGroup m_ugFilter;
   bool m_bSaveNeeded;

   // Methods
   public:
   bool Add(const CUser& user);
   bool Add(const CString& strName, const CString& strPassword, 
         TUserGroup ug, unsigned int nTimeout, unsigned int nRemoteAccess);
   bool Remove(const CString& strUser);
   bool SetFilter(TUserGroup ug);
   void RemoveFilter();
   bool GetUser(CUser& u, int i);
   bool GetUser(CUser& u, CString strUser);
   bool UpdateUser(const CString& strUser, const CString& strName, 
         const CString& strPassword, TUserGroup ug, unsigned int nTimeout, unsigned int nRemoteAccess);
   int GetUserCount();
   bool IsValidUserName(const CString& str);
   bool IsValidPassword(const CString& str);
   bool IsValidUserGroup(int ug);
   bool Commit();
   void LoginPort(const CString& strUser, const CString& strPort);
   TTimeSpan LogoutPort(const CString& strUser, const CString& strPort);
   // This is a list of logged in users, which includes the device they logged in on, and the time they logged in
   // To access the list directly, minimally a read lock must be obtained using m_LoginPortLock
   TUtils_MapStringToRefUpperCase<TLoginPort> m_LoginPort;
   TRWLock m_LoginPortLock;

   protected:
   bool GetUserPtr(CUser*& u, CString strUser);
};


// Extern for global users object
extern "C" DECL CUsers Users;

#endif //_Users_H_

